"use strict";
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
const Moment = __importStar(require("moment"));
const moment_range_1 = require("moment-range");
const utils_1 = require("ethers/utils");
const origin_backend_core_1 = require("@energyweb/origin-backend-core");
const PreciseProofEntity_1 = require("./PreciseProofEntity");
const contracts_1 = require("../contracts");
const events_1 = require("../utils/events");
class CertificationRequest extends PreciseProofEntity_1.PreciseProofEntity {
    constructor(id, configuration, isPrivate = false) {
        super(id, configuration);
        this.isPrivate = isPrivate;
        this.initialized = false;
    }
    static async create(fromTime, toTime, energy, deviceId, configuration, files, isVolumePrivate = false, forAddress) {
        if (energy.gt(origin_backend_core_1.MAX_ENERGY_PER_CERTIFICATE)) {
            throw new Error(`Too much energy requested. Requested: ${energy}, Max: ${origin_backend_core_1.MAX_ENERGY_PER_CERTIFICATE}`);
        }
        const request = new CertificationRequest(null, configuration, isVolumePrivate);
        const { issuer } = configuration.blockchainProperties;
        const issuerWithSigner = issuer.connect(configuration.blockchainProperties.activeUser);
        await this.validateGenerationPeriod(fromTime, toTime, deviceId, configuration, isVolumePrivate);
        const data = await issuer.encodeData(fromTime, toTime, deviceId);
        const tx = await (forAddress
            ? issuerWithSigner.requestCertificationFor(data, forAddress, isVolumePrivate)
            : issuerWithSigner.requestCertification(data, isVolumePrivate));
        const { events } = await tx.wait();
        request.id = Number(events.find((log) => log.event === 'CertificationRequested').topics[2]);
        const success = await configuration.offChainDataSource.certificateClient.updateCertificationRequestData(request.id, { energy, files });
        if (success) {
            if (configuration.logger) {
                configuration.logger.info(`CertificationRequest ${request.id} created`);
            }
        }
        else {
            throw new Error('Unable to create CertificationRequest');
        }
        return request.sync();
    }
    async sync() {
        const { issuer } = this.configuration
            .blockchainProperties;
        const issueRequest = await issuer.getCertificationRequest(this.id);
        const decodedData = await issuer.decodeData(issueRequest.data);
        this.owner = issueRequest.owner;
        this.fromTime = Number(decodedData['0']);
        this.toTime = Number(decodedData['1']);
        this.deviceId = decodedData['2'];
        this.approved = issueRequest.approved;
        this.revoked = issueRequest.revoked;
        const certificationRequestedLogs = await events_1.getEventsFromContract(issuer, issuer.filters.CertificationRequested(null, this.id, null));
        const creationBlock = await issuer.provider.getBlock(certificationRequestedLogs[0].blockNumber);
        this.created = Number(creationBlock.timestamp);
        const offChainData = await this.configuration.offChainDataSource.certificateClient.getCertificationRequestData(this.id);
        this.energy = offChainData.energy;
        this.files = offChainData.files;
        this.initialized = true;
        if (this.configuration.logger) {
            this.configuration.logger.verbose(`CertificationRequest ${this.id} synced`);
        }
        return this;
    }
    async approve() {
        const issuerInterface = new utils_1.Interface(contracts_1.IssuerJSON.abi);
        const validityData = issuerInterface.functions.isRequestValid.encode([this.id.toString()]);
        let approveTx;
        const { issuer } = this.configuration
            .blockchainProperties;
        const issuerWithSigner = issuer.connect(this.configuration.blockchainProperties.activeUser);
        if (this.isPrivate) {
            const commitment = {
                [this.owner]: this.energy
            };
            const { rootHash } = this.generateAndAddProofs(commitment);
            approveTx = await issuerWithSigner.approveCertificationRequestPrivate(this.id, rootHash, validityData);
        }
        else {
            approveTx = await issuerWithSigner.approveCertificationRequest(this.id, this.energy, validityData);
        }
        const { events } = await approveTx.wait();
        const certificateId = Number(events.find((log) => log.event === 'CertificationRequestApproved')
            .topics[3]);
        return certificateId;
    }
    async revoke() {
        const { issuer } = this.configuration
            .blockchainProperties;
        const issuerWithSigner = issuer.connect(this.configuration.blockchainProperties.activeUser);
        const revokeTx = await issuerWithSigner.revokeRequest(this.id);
        await revokeTx.wait();
    }
    async requestMigrateToPublic(value) {
        if (!this.isPrivate) {
            throw new Error('Certificate is already public.');
        }
        const { issuer } = this.configuration
            .blockchainProperties;
        const activeUserAddress = await this.configuration.blockchainProperties.activeUser.getAddress();
        const commitment = {
            [activeUserAddress]: value
        };
        const { rootHash } = this.generateAndAddProofs(commitment);
        const issuerWithSigner = issuer.connect(this.configuration.blockchainProperties.activeUser);
        const tx = await issuerWithSigner.requestMigrateToPublic(this.id, rootHash);
        await tx.wait();
        return tx;
    }
    static async validateGenerationPeriod(fromTime, toTime, deviceId, configuration, isPrivate) {
        const { issuer } = configuration.blockchainProperties;
        const moment = moment_range_1.extendMoment(Moment);
        const unix = (timestamp) => moment.unix(timestamp);
        const certificationRequestedEvents = await events_1.getEventsFromContract(issuer, issuer.filters.CertificationRequested(null, null, deviceId));
        const certificationRequestIds = certificationRequestedEvents.map((event) => event._id);
        const generationTimeRange = moment.range(unix(fromTime), unix(toTime));
        for (const id of certificationRequestIds) {
            const certificationRequest = await new CertificationRequest(id.toNumber(), configuration, isPrivate).sync();
            if (certificationRequest.revoked) {
                continue;
            }
            const certificationRequestGenerationRange = moment.range(unix(certificationRequest.fromTime), unix(certificationRequest.toTime));
            if (generationTimeRange.overlaps(certificationRequestGenerationRange)) {
                throw new Error(`Generation period ` +
                    `${unix(fromTime).format()} - ${unix(toTime).format()}` +
                    ` overlaps with the time period of ` +
                    `${unix(certificationRequest.fromTime).format()} - ${unix(certificationRequest.toTime).format()}` +
                    ` of request ${certificationRequest.id}.`);
            }
        }
        return true;
    }
}
exports.CertificationRequest = CertificationRequest;
async function getAllCertificationRequests(configuration) {
    const { issuer } = configuration.blockchainProperties;
    const totalRequests = await issuer.totalRequests();
    const certificationRequestPromises = Array(Number(totalRequests))
        .fill(null)
        .map(async (item, index) => new CertificationRequest(index + 1, configuration).sync());
    return Promise.all(certificationRequestPromises);
}
exports.getAllCertificationRequests = getAllCertificationRequests;
//# sourceMappingURL=CertificationRequest.js.map