"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const chai_1 = require("chai");
const dotenv_1 = __importDefault(require("dotenv"));
require("mocha");
const moment_1 = __importDefault(require("moment"));
const utils_1 = require("ethers/utils");
const origin_backend_client_mocks_1 = require("@energyweb/origin-backend-client-mocks");
const ethers_1 = require("ethers");
const migrate_1 = require("../migrate");
const __1 = require("..");
const Logger_1 = require("../Logger");
describe('Issuer', () => {
    let conf;
    dotenv_1.default.config({
        path: '.env.test'
    });
    const provider = new ethers_1.providers.JsonRpcProvider(process.env.WEB3);
    const deviceOwnerPK = '0x622d56ab7f0e75ac133722cc065260a2792bf30ea3265415fe04f3a2dba7e1ac';
    const deviceOwnerWallet = new ethers_1.Wallet(deviceOwnerPK, provider);
    const issuerPK = '0x50397ee7580b44c966c3975f561efb7b58a54febedaa68a5dc482e52fb696ae7';
    const issuerWallet = new ethers_1.Wallet(issuerPK, provider);
    let timestamp = moment_1.default().subtract(10, 'year').unix();
    const setActiveUser = (wallet) => {
        conf.blockchainProperties.activeUser = wallet;
    };
    const createCertificationRequest = async (energy, isPrivate = false) => {
        setActiveUser(deviceOwnerWallet);
        const fromTime = timestamp;
        // Simulate time moving forward 1 month
        timestamp += 30 * 24 * 3600;
        const toTime = timestamp;
        const device = '1';
        return __1.CertificationRequest.create(fromTime, toTime, energy, device, conf, [], isPrivate);
    };
    it('migrates Issuer and Registry', async () => {
        const registry = await migrate_1.migrateRegistry(process.env.WEB3, issuerPK);
        const issuer = await migrate_1.migrateIssuer(process.env.WEB3, issuerPK, registry.address);
        const version = await issuer.version();
        chai_1.assert.equal(version, 'v0.1');
        const registryAddress = await issuer.getRegistryAddress();
        chai_1.assert.equal(registryAddress, registry.address);
        conf = {
            blockchainProperties: {
                activeUser: issuerWallet,
                registry,
                issuer
            },
            offChainDataSource: new origin_backend_client_mocks_1.OffChainDataSourceMock(),
            logger: Logger_1.logger
        };
    });
    it('user correctly requests issuance', async () => {
        setActiveUser(deviceOwnerWallet);
        const energy = new utils_1.BigNumber(1e9);
        const fromTime = timestamp;
        // Simulate time moving forward 1 month
        timestamp += 30 * 24 * 3600;
        const toTime = timestamp;
        const device = '1';
        const certificationRequest = await __1.CertificationRequest.create(fromTime, toTime, energy, device, conf, []);
        chai_1.assert.isAbove(certificationRequest.id, -1);
        chai_1.assert.deepOwnInclude(certificationRequest, {
            initialized: true,
            deviceId: device,
            owner: deviceOwnerWallet.address,
            fromTime,
            toTime,
            approved: false,
            isPrivate: false,
            energy
        });
    });
    it('issuer correctly approves issuance', async () => {
        const volume = new utils_1.BigNumber(1e9);
        let certificationRequest = await createCertificationRequest(volume);
        setActiveUser(issuerWallet);
        const certificateId = await certificationRequest.approve();
        certificationRequest = await certificationRequest.sync();
        chai_1.assert.isTrue(certificationRequest.approved);
        const deviceOwnerBalance = await conf.blockchainProperties.registry.balanceOf(deviceOwnerWallet.address, certificateId);
        chai_1.assert.equal(deviceOwnerBalance.toString(), volume.toString());
    });
    it('user revokes a certificationRequest', async () => {
        setActiveUser(deviceOwnerWallet);
        const volume = new utils_1.BigNumber(1e9);
        let certificationRequest = await createCertificationRequest(volume);
        certificationRequest = await certificationRequest.sync();
        chai_1.assert.isFalse(certificationRequest.approved);
        chai_1.assert.isFalse(certificationRequest.revoked);
        await certificationRequest.revoke();
        certificationRequest = await certificationRequest.sync();
        chai_1.assert.isTrue(certificationRequest.revoked);
    });
    it('user shouldnt be able to revoke an approved certificationRequest', async () => {
        setActiveUser(deviceOwnerWallet);
        const volume = new utils_1.BigNumber(1e9);
        let certificationRequest = await createCertificationRequest(volume);
        setActiveUser(issuerWallet);
        certificationRequest = await certificationRequest.sync();
        await certificationRequest.approve();
        setActiveUser(deviceOwnerWallet);
        certificationRequest = await certificationRequest.sync();
        chai_1.assert.isTrue(certificationRequest.approved);
        chai_1.assert.isFalse(certificationRequest.revoked);
        let failed = false;
        try {
            await certificationRequest.revoke();
        }
        catch (e) {
            failed = true;
        }
        chai_1.assert.isTrue(failed);
    });
    it('should fail to request 2 certificates with the same generation period', async () => {
        setActiveUser(deviceOwnerWallet);
        const fromTime = timestamp;
        // Simulate time moving forward 1 month
        timestamp += 30 * 24 * 3600;
        const toTime = timestamp;
        const device = '1';
        await __1.CertificationRequest.create(fromTime, toTime, new utils_1.BigNumber(1e9), device, conf, []);
        let failed = false;
        try {
            await __1.CertificationRequest.create(fromTime, toTime, new utils_1.BigNumber(1e9), device, conf, []);
        }
        catch (e) {
            failed = true;
        }
        chai_1.assert.isTrue(failed);
    });
    it('should request the same certificate after revoking one', async () => {
        setActiveUser(deviceOwnerWallet);
        const fromTime = timestamp;
        // Simulate time moving forward 1 month
        timestamp += 30 * 24 * 3600;
        const toTime = timestamp;
        const device = '1';
        const volume = new utils_1.BigNumber(1e9);
        let certificationRequest = await __1.CertificationRequest.create(fromTime, toTime, volume, device, conf, []);
        certificationRequest = await certificationRequest.sync();
        await certificationRequest.revoke();
        certificationRequest = await certificationRequest.sync();
        const newCertificationRequest = await __1.CertificationRequest.create(fromTime, toTime, volume, device, conf, []);
        chai_1.assert.exists(newCertificationRequest);
    });
    it('user correctly requests private issuance', async () => {
        const volume = new utils_1.BigNumber(1e9);
        const certificationRequest = await createCertificationRequest(volume, true);
        chai_1.assert.isAbove(Number(certificationRequest.id), -1);
        chai_1.assert.deepOwnInclude(certificationRequest, {
            initialized: true,
            deviceId: '1',
            owner: deviceOwnerWallet.address,
            approved: false,
            isPrivate: true,
            energy: volume
        });
    });
    it('issuer correctly approves private issuance', async () => {
        const volume = new utils_1.BigNumber(1e9);
        let certificationRequest = await createCertificationRequest(volume, true);
        setActiveUser(issuerWallet);
        const certificateId = await certificationRequest.approve();
        certificationRequest = await certificationRequest.sync();
        chai_1.assert.isTrue(certificationRequest.approved);
        const deviceOwnerBalance = await conf.blockchainProperties.registry.balanceOf(deviceOwnerWallet.address, Number(certificateId));
        chai_1.assert.equal(deviceOwnerBalance.toString(), '0');
    });
});
//# sourceMappingURL=Issuer.test.js.map